/*PS3Controller.cpp
Purpose: Contains structures and functions essential to the execution of PS3Interface.cpp
Author: Justin Ashtiani, Colin Usher
Version 1.0 07/17/2013
*/

#include "StdAfx.h"
#include "PS3Controller.h"


PS3Controller::PS3Controller(void)//constructor
{
	g_pDI = NULL;
    g_pJoystick = NULL;
	Initialize();
}


PS3Controller::~PS3Controller(void)//destructor
{
	FreeDirectInput();
	UpdateInputState();
}



HRESULT PS3Controller::Initialize()
{
    HRESULT hr;

    // Register with the DirectInput subsystem and get a pointer
    // to a IDirectInput interface we can use.
    // Create a DInput object
    if( FAILED( hr = DirectInput8Create( GetModuleHandle( NULL ), DIRECTINPUT_VERSION,
                                         IID_IDirectInput8, ( VOID** )&g_pDI, NULL ) ) )
        return hr;

	DIJOYCONFIG PreferredJoyCfg = {0};
    DI_ENUM_CONTEXT enumContext;
    enumContext.pPreferredJoyCfg = &PreferredJoyCfg;
    enumContext.bPreferredJoyCfgValid = false;

////////////////////////////////////////////////////////
    IDirectInputJoyConfig8* pJoyConfig = NULL;
    if( FAILED( hr = g_pDI->QueryInterface( IID_IDirectInputJoyConfig8, ( void** )&pJoyConfig ) ) )
        return hr;

    PreferredJoyCfg.dwSize = sizeof( PreferredJoyCfg );
    if( SUCCEEDED( pJoyConfig->GetConfig( 0, &PreferredJoyCfg, DIJC_GUIDINSTANCE ) ))
		
		std::cout << "PS3 Controller Connected" << "\n";
		std::cout << "First value outputted is junk. Please ignore." << "\n";
		
		 // This function is expected to fail if no joystick is attached
        enumContext.bPreferredJoyCfgValid = true;
    SAFE_RELEASE( pJoyConfig );
	
///////////////////////////////////////////////////////////

	    if( FAILED( hr = g_pDI->EnumDevices(DI8DEVCLASS_GAMECTRL, &PS3Controller::enumJoysticksTrampoline,						this ,DIEDFL_ATTACHEDONLY ) ) )
        return hr;

    // Set the data format to "simple joystick" - a predefined data format 
    //
    // A data format specifies which controls on a device we are interested in,
    // and how they should be reported. This tells DInput that we will be
    // passing a DIJOYSTATE2 structure to IDirectInputDevice::GetDeviceState().
    if( FAILED( hr = g_pJoystick->SetDataFormat( &c_dfDIJoystick2 ) ) )
        return hr;

    // Set the cooperative level to let DInput know how this device should
    // interact with the system and with other DInput applications.
    if( FAILED( hr = g_pJoystick->SetCooperativeLevel( ::GetDesktopWindow(), DISCL_BACKGROUND |	
														DISCL_NONEXCLUSIVE ) ) )
        return hr;

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: EnumJoysticksCallback()
// Desc: Called once for each enumerated joystick. If we find one, create a
//       device interface on it so we can play with it.
//-----------------------------------------------------------------------------
BOOL CALLBACK PS3Controller::EnumJoysticksCallback( const DIDEVICEINSTANCE* pdidInstance,
                                     VOID* pContext )
{
	
    DI_ENUM_CONTEXT* pEnumContext = ( DI_ENUM_CONTEXT* )pContext;
    HRESULT hr;

    //if( g_bFilterOutXinputDevices && IsXInputDevice( &pdidInstance->guidProduct ) )
    //    return DIENUM_CONTINUE;

    // Skip anything other than the perferred joystick device as defined by the control panel.  
    // Instead you could store all the enumerated joysticks and let the user pick.
    if( pEnumContext->bPreferredJoyCfgValid &&
        !IsEqualGUID( pdidInstance->guidInstance, pEnumContext->pPreferredJoyCfg->guidInstance ) )
        return DIENUM_CONTINUE;

    // Obtain an interface to the enumerated joystick.
    hr = g_pDI->CreateDevice( pdidInstance->guidInstance, &g_pJoystick, NULL );

    // If it failed, then we can't use this joystick. (Maybe the user unplugged
    // it while we were in the middle of enumerating it.)
    if( FAILED( hr ) )
        return DIENUM_CONTINUE;

    // Stop enumeration. Note: we're just taking the first joystick we get. You
    // could store all the enumerated joysticks and let the user pick.
    return DIENUM_STOP;
}

HRESULT PS3Controller::UpdateInputState( )
{
	HRESULT hr;//data type for error/warning conditions
	TCHAR strText[512] = {0}; // Device state text
	DIJOYSTATE2 js;           // DInput joystick state 

	if( NULL == g_pJoystick )
		return S_OK;

	// Poll the device to read the current state
	hr = g_pJoystick->Poll();
	if( FAILED( hr ) )
	{
		// DInput is telling us that the input stream has been
		// interrupted. We aren't tracking any state between polls, so
		// we don't have any special reset that needs to be done. We
		// just re-acquire and try again.
		hr = g_pJoystick->Acquire();
		while( hr == DIERR_INPUTLOST )
			hr = g_pJoystick->Acquire();

		// hr may be DIERR_OTHERAPPHASPRIO or other errors.  This
		// may occur when the app is minimized or in the process of 
		// switching, so just try again later 
		//return S_OK;
	}


// Get the input's device state
    if( FAILED( hr = g_pJoystick->GetDeviceState( sizeof( DIJOYSTATE2 ), &js ) ) )
        return hr; // The device should have been acquired during the Poll()

//Define direction_axis -> js.l(direction)(axis);
	myCS.left_y =  js.lY;
	myCS.left_x = js.lX;
	myCS.right_y = js.lRy;
	myCS.right_x = js.lRx;

	//std::cout << myCS.left_y << " " << myCS.right_y << "\n";

	return S_OK;//Return success code
}

//-----------------------------------------------------------------------------
// Name: FreeDirectInput()
// Desc: Initialize the DirectInput variables.
//-----------------------------------------------------------------------------
VOID PS3Controller::FreeDirectInput()
{
    // Unacquire the device one last time just in case 
    // the app tried to exit while the device is still acquired.
    if( g_pJoystick )
        g_pJoystick->Unacquire();

    // Release any DirectInput objects.
    SAFE_RELEASE( g_pJoystick );
    SAFE_RELEASE( g_pDI );
}


int PS3Controller::getControllerAnalogPosScaled(int which)//determine which joystick input you would like
{
	//number for each of the following //1 = lx 65535 is highest x, 2 = ly 32511, 3 = rx, 4 = ry
	switch (which)
	{
		std::cout << "here";
	case(1):
		return myCS.left_x/256;
		break;
	case(2):
		//std::cout << myCS.left_y << "  " << myCS.left_y/256 << "\n";
		return myCS.left_y/256;
		break;
	case(3):
		return myCS.right_x/256;
		break;
	case (4):
		//std::cout << myCS.right_y << "  " << myCS.right_y/256 << "\n";
		return myCS.right_y/256;
		break;
	}

}

int PS3Controller::scaleAnalogPos(int pos)//scale the value to work with the robot
{
	if(pos= myCS.left_x)
	{
		return pos/= 255;//take raw joystick value and convert to value compatible with robot
	}
	
	else
	{
		return pos/= 255;
	}

}