/*PS3Controller.h
Purpose: Contains structures and classes used by PS3Controller.cpp and Ps3Interface.cpp
Author: Justin Ashtiani, Colin Usher
Version 1.0 07/17/2013
*/

#pragma once

#define STRICT
#define DIRECTINPUT_VERSION 0x0800
#define _CRT_SECURE_NO_DEPRECATE
#ifndef _WIN32_DCOM
#define _WIN32_DCOM
#endif

#include "stdafx.h"//include file for standard system include files, project specific include files
#include <stdio.h>//C Standard Input and Output Library
#include <stdlib.h>//general purpose functions including dynamic memory management, random number generation, environment communication, integer arithmetics, searching, sorting, converting
#include <iostream>//header that defines standard input/output stream objects
#include <InitGuid.h>//System and driver specific header files that contain GUID definitions

#include <windows.h>//contains functions related to programming in Windows, essentially the WinAPI
#include <stdio.h>//input and output operations can be performed using stdio.h and streams to operate devices
#include <tchar.h>//unicode mapping layer that can be used to adapt to unicode and non-unicode environments
#include <commctrl.h>//functions and toolbars used to create and manage a toolbar
#include <basetsd.h>//Used to define new data types and macros in 32 and 64 bit environments

#pragma warning(push)
#pragma warning(disable:6000 28251)
#include <dinput.h>//part of the DirectX SDK
#pragma warning(pop)

#include <dinputd.h>//Direct input header file
#include <assert.h>//defines a macro function that can be used as a standard debugging tool
#include <oleauto.h>//allows for OLE Automation
#include <shellapi.h>//Windows Shell API


//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
#define SAFE_DELETE(p)  { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

//-----------------------------------------------------------------------------
// STRUCTURES
//-----------------------------------------------------------------------------
struct DI_ENUM_CONTEXT
{
    DIJOYCONFIG* pPreferredJoyCfg;
    bool bPreferredJoyCfgValid;
};

struct controllerStatus
{
	int left_y;
	int left_x;
	int right_x;
	int right_y;
	int pos; 
};


class PS3Controller
{

public:

	//Variables
	LPDIRECTINPUT8          g_pDI;
	LPDIRECTINPUTDEVICE8    g_pJoystick;

	//Functions
	PS3Controller(void); 
	~PS3Controller(void);
	controllerStatus PS3Controller::myCS;
	

	int scaleAnalogPos(int pos);  //take in 0-65535, spit out 0 - 255
	int getControllerAnalogPosScaled(int which);  //1 = lx, 2 = ly, 3 = rx, 4 = ry
	HRESULT UpdateInputState( );
	HRESULT Initialize();
	VOID FreeDirectInput();

	//
	BOOL CALLBACK EnumJoysticksCallback( const DIDEVICEINSTANCE* pdidInstance, VOID* pContext );
	
private:


	//controllerStatus PS3Controller::myCS;
	static BOOL CALLBACK enumJoysticksTrampoline(
	  LPCDIDEVICEINSTANCE lpddi,
	  LPVOID pvRef
	  )
	 {
	  return reinterpret_cast<PS3Controller*>(pvRef)
	   ->EnumJoysticksCallback(lpddi, pvRef);
	 }
};

